from typing import Annotated
from typing import Any
from typing import cast

import structlog
import typer

from ..protocol.local import LocalSession
from ..protocol.rdp import RDPSession
from ..protocol.vnc import VNCSession
from .server import mcp
from .server import set_session

logger = structlog.get_logger()

app = typer.Typer(
    help='Start the MCP server for various protocols.',
    no_args_is_help=True,
)


@app.command(name='local')
def mcp_local(
    display: Annotated[
        str | None,
        typer.Option(
            help='Display number (e.g. :0).',
            envvar='REMOTE_DESKTOP_DISPLAY',
        ),
    ] = None,
):
    """Start MCP server for the local desktop."""
    logger.info('starting_local_mcp_server', display=display)
    session = LocalSession(display_num=display)
    set_session(session)
    try:
        mcp.run()
    finally:
        session.stop()


@app.command(name='rdp')
def mcp_rdp(
    host: Annotated[
        str,
        typer.Option(
            help='RDP remote host address',
            envvar='REMOTE_DESKTOP_RDP_HOST',
        ),
    ] = cast(Any, ...),
    user: Annotated[
        str,
        typer.Option(help='RDP username', envvar='REMOTE_DESKTOP_RDP_USER'),
    ] = cast(Any, ...),
    password: Annotated[
        str,
        typer.Option(
            help='RDP password',
            envvar='REMOTE_DESKTOP_RDP_PASSWORD',
        ),
    ] = cast(Any, ...),
    display: Annotated[
        str,
        typer.Option(
            help='Local Xvfb display number.',
            envvar='REMOTE_DESKTOP_DISPLAY',
        ),
    ] = ':99',
    resolution: Annotated[
        str,
        typer.Option(
            help='Screen resolution.',
            envvar='REMOTE_DESKTOP_RESOLUTION',
        ),
    ] = '1280x720',
):
    """Start MCP server for an RDP connection."""
    logger.info(
        'starting_rdp_mcp_server', host=host,
        user=user, display=display,
    )
    session = RDPSession(
        host=host, user=user, password=password,
        display_num=display, resolution=resolution,
    )
    set_session(session)
    try:
        mcp.run()
    finally:
        session.stop()


@app.command(name='vnc')
def mcp_vnc(
    host: Annotated[
        str,
        typer.Option(
            help='VNC remote host address (host:port)',
            envvar='REMOTE_DESKTOP_VNC_HOST',
        ),
    ] = cast(Any, ...),
    password: Annotated[
        str | None,
        typer.Option(
            help='VNC password',
            envvar='REMOTE_DESKTOP_VNC_PASSWORD',
        ),
    ] = None,
):
    """Start MCP server for a VNC connection."""
    logger.info('starting_vnc_mcp_server', host=host)
    session = VNCSession(host=host, password=password)
    set_session(session)
    try:
        mcp.run()
    finally:
        session.stop()


if __name__ == '__main__':
    app()



import os

import structlog
from mss import mss

from .base import GUISession
from .base import Key
from .base import MouseButton

logger = structlog.get_logger()


class LocalSession(GUISession):
    display_num: str | None

    def __init__(self, display_num: str | None = None) -> None:
        self.display_num = display_num

    def _set_display_env(self) -> None:
        if self.display_num:
            os.environ['DISPLAY'] = self.display_num
        elif 'DISPLAY' not in os.environ:
            os.environ['DISPLAY'] = ':0'

    def start(self) -> None:
        logger.info('local_session_ready', protocol='local')

    def stop(self) -> None:
        pass

    def take_screenshot(self, output_path: str) -> None:
        logger.info('screenshot', protocol='local', path=output_path)
        with mss(display=self.display_num) as sct:
            sct.shot(output=output_path)

    def click(self, x: int, y: int, button: MouseButton = MouseButton.LEFT) -> None:
        logger.info('click', protocol='local', x=x, y=y, button=button.value)
        self._set_display_env()
        import pyautogui
        pyautogui.click(x=x, y=y, button=button.value)

    def move(self, x: int, y: int) -> None:
        logger.info('move', protocol='local', x=x, y=y)
        self._set_display_env()
        import pyautogui
        pyautogui.moveTo(x, y)

    def scroll(self, clicks: int) -> None:
        logger.info('scroll', protocol='local', clicks=clicks)
        self._set_display_env()
        import pyautogui
        pyautogui.scroll(clicks)

    def type_text(self, text: str) -> None:
        logger.info('type_text', protocol='local', text=text)
        self._set_display_env()
        import pyautogui
        pyautogui.write(text)

    def key_event(self, key: Key) -> None:
        logger.info('key_event', protocol='local', key=key.value)
        self._set_display_env()
        import pyautogui
        pyautogui.press(key.value)



from typing import Annotated
from typing import Any
from typing import cast

import structlog
import typer

from ..protocol.base import MouseButton
from ..protocol.local import LocalSession
from ..protocol.rdp import RDPSession
from ..protocol.vnc import VNCSession

logger = structlog.get_logger()

app = typer.Typer(
    help='Remote Desktop Debug CLI - Test desktop protocols and actions independently.',
    no_args_is_help=True,
)

# Local Debug
debug_local_app = typer.Typer(help='Debug local desktop.')
app.add_typer(debug_local_app, name='local')


@debug_local_app.command(name='screenshot')
def debug_local_screenshot(
    output: Annotated[
        str, typer.Option(
        help='Output path',
        ),
    ] = 'screenshot.png',
    display: Annotated[
        str | None,
        typer.Option(help='Display number', envvar='REMOTE_DESKTOP_DISPLAY'),
    ] = None,
):
    session = LocalSession(display_num=display)
    session.start()
    session.take_screenshot(output)
    print(f"Screenshot saved to {output}")
    session.stop()


@debug_local_app.command(name='click')
def debug_local_click(
    x: int, y: int,
    button: MouseButton = MouseButton.LEFT,
    display: Annotated[
        str | None,
        typer.Option(help='Display number', envvar='REMOTE_DESKTOP_DISPLAY'),
    ] = None,
):
    session = LocalSession(display_num=display)
    session.start()
    session.click(x, y, button)
    session.stop()


# VNC Debug
debug_vnc_app = typer.Typer(help='Debug VNC connection.')
app.add_typer(debug_vnc_app, name='vnc')


@debug_vnc_app.command(name='screenshot')
def debug_vnc_screenshot(
    host: Annotated[
        str,
        typer.Option(help='VNC host', envvar='REMOTE_DESKTOP_VNC_HOST'),
    ] = cast(Any, ...),
    password: Annotated[
        str | None,
        typer.Option(
            help='VNC password',
            envvar='REMOTE_DESKTOP_VNC_PASSWORD',
        ),
    ] = None,
    output: str = 'vnc_screenshot.png',
):
    session = VNCSession(host=host, password=password)
    session.start()
    session.take_screenshot(output)
    print(f"Screenshot saved to {output}")
    session.stop()


# RDP Debug
debug_rdp_app = typer.Typer(help='Debug RDP connection.')
app.add_typer(debug_rdp_app, name='rdp')


@debug_rdp_app.command(name='screenshot')
def debug_rdp_screenshot(
    host: Annotated[
        str,
        typer.Option(help='RDP host', envvar='REMOTE_DESKTOP_RDP_HOST'),
    ] = cast(Any, ...),
    user: Annotated[
        str,
        typer.Option(help='RDP user', envvar='REMOTE_DESKTOP_RDP_USER'),
    ] = cast(Any, ...),
    password: Annotated[
        str,
        typer.Option(
            help='RDP password',
            envvar='REMOTE_DESKTOP_RDP_PASSWORD',
        ),
    ] = cast(Any, ...),
    output: str = 'rdp_screenshot.png',
    display: Annotated[
        str,
        typer.Option(help='Display number', envvar='REMOTE_DESKTOP_DISPLAY'),
    ] = ':99',
    resolution: Annotated[
        str,
        typer.Option(help='Resolution', envvar='REMOTE_DESKTOP_RESOLUTION'),
    ] = '1280x720',
):
    session = RDPSession(
        host, user, password,
        display_num=display, resolution=resolution,
    )
    session.start()
    session.take_screenshot(output)
    print(f"Screenshot saved to {output}")
    session.stop()


if __name__ == '__main__':
    app()



import structlog
import typer

from .cli.__main__ import app as cli_app
from .mcp.__main__ import app as mcp_app

# Configure structlog
structlog.configure(
    processors=[
        structlog.processors.add_log_level,
        structlog.processors.TimeStamper(fmt='iso'),
        structlog.processors.JSONRenderer(),
    ],
)

app = typer.Typer(
    help='Remote Desktop MCP - Manage and debug remote desktop connections via MCP or CLI.',
    no_args_is_help=True,
)

app.add_typer(mcp_app, name='mcp')
app.add_typer(cli_app, name='cli')

if __name__ == '__main__':
    app()



from fastmcp import FastMCP

from ..protocol.base import GUISession
from ..protocol.base import Key
from ..protocol.base import MouseButton

mcp = FastMCP('Remote Desktop MCP')

# Global session instance
_session: GUISession | None = None


def set_session(session: GUISession) -> None:
    global _session
    _session = session
    _session.start()


@mcp.tool()
def take_screenshot(output_path: str = 'screenshot.png') -> str:
    """Capture a screenshot of the remote desktop."""
    if not _session:
        return 'Error: No session active.'
    _session.take_screenshot(output_path)
    return f"Screenshot saved to {output_path}"


@mcp.tool()
def mouse_click(x: int, y: int, button: str = 'left') -> str:
    """Click at the specified coordinates (x, y)."""
    if not _session:
        return 'Error: No session active.'
    try:
        btn = MouseButton(button.lower())
    except ValueError:
        return f"Error: Invalid button '{button}'. Use 'left', 'middle', or 'right'."
    _session.click(x, y, btn)
    return f"Clicked {button} at ({x}, {y})"


@mcp.tool()
def mouse_move(x: int, y: int) -> str:
    """Move the mouse cursor to (x, y)."""
    if not _session:
        return 'Error: No session active.'
    _session.move(x, y)
    return f"Moved mouse to ({x}, {y})"


@mcp.tool()
def mouse_scroll(clicks: int) -> str:
    """Scroll the mouse wheel. Positive for up, negative for down."""
    if not _session:
        return 'Error: No session active.'
    _session.scroll(clicks)
    return f"Scrolled {clicks} clicks"


@mcp.tool()
def type_text(text: str) -> str:
    """Type the specified text."""
    if not _session:
        return 'Error: No session active.'
    _session.type_text(text)
    return f"Typed: {text}"


@mcp.tool()
def press_key(key_name: str) -> str:
    """Press a special key (e.g., enter, backspace, tab, space, esc, up, down, left, right)."""
    if not _session:
        return 'Error: No session active.'
    try:
        key = Key(key_name.lower())
    except ValueError:
        valid_keys = ', '.join([k.value for k in Key])
        return f"Error: Invalid key '{key_name}'. Valid keys: {valid_keys}"
    _session.key_event(key)
    return f"Pressed key: {key_name}"
